@@ -1,3 +1,4 @@
+import itertools
 from pathlib import Path
 
 from open_dread_rando import lua_util
@@ -44,10 +45,14 @@ class LuaEditor:
     def get_parent_for(self, item_id) -> str:
         return SPECIFIC_CLASSES.get(item_id, "RandomizerPowerup")
 
-    def get_script_class(self, pickup_resources: list[dict], boss: bool = False) -> str:
+    def get_script_class(self, pickup: dict, boss: bool = False, actordef_name: str = "") -> str:
+        pickup_resources = pickup["resources"]
         parent = self.get_parent_for(pickup_resources[0]["item_id"])
         if not boss and len(pickup_resources) == 1:
             return parent
+        
+        if actordef_name and len(pickup["model"]) > 1:
+            self.add_progressive_models(pickup, actordef_name)
 
         hashable_progression = "_".join([f'{res["item_id"]}_{res["quantity"]}' for res in pickup_resources])
 
@@ -76,8 +81,29 @@ class LuaEditor:
     def add_progressive_class(self, replacement):
         new_class = lua_util.replace_lua_template("randomizer_progressive_template.lua", replacement)
         self._powerup_script += new_class.encode("utf-8")
+    
+    def add_progressive_models(self, pickup: dict, actordef_name: str):
+        progressive_models = [
+            {
+                "item": lua_util.wrap_string(resource["item_id"]),
+                "alias": lua_util.wrap_string(model_name),
+            }
+            for resource, model_name in itertools.chain(
+                zip(pickup["resources"], pickup["model"][1:]),
+                [(pickup["resources"][-1], pickup["model"][-1])],
+            )
+        ]
+        progressive_models.reverse()
 
-    def patch_actordef_pickup_script(self, editor: PatcherEditor, resources: list[dict], pickup_lua_callback: dict,
+        replacement = {
+            "actordef_name": actordef_name,
+            "progressive_models": progressive_models,
+        }
+
+        models = lua_util.replace_lua_template("progressive_model_template.lua", replacement)
+        self._powerup_script += models.encode("utf-8")
+
+    def patch_actordef_pickup_script(self, editor: PatcherEditor, pickup: dict, pickup_lua_callback: dict,
                                      extra_code: str = ""):
         scenario = pickup_lua_callback["scenario"]
         scenario_path = path_for_level(scenario)
@@ -92,15 +118,15 @@ class LuaEditor:
         replacement = {
             "scenario": scenario,
             "funcname": pickup_lua_callback["function"],
-            "pickup_class": self.get_script_class(resources, True),
+            "pickup_class": self.get_script_class(pickup, True),
             "extra_code": extra_code,
             "args": ", ".join([f"_ARG_{i}_" for i in range(pickup_lua_callback["args"])])
         }
         script["script"] += lua_util.replace_lua_template("boss_powerup_template.lua", replacement)
 
-    def patch_corex_pickup_script(self, editor: PatcherEditor, resources: list[dict], pickup_lua_callback: dict):
+    def patch_corex_pickup_script(self, editor: PatcherEditor, pickup: dict, pickup_lua_callback: dict):
         bossid = pickup_lua_callback["function"]
-        self._corex_replacement[bossid] = self.get_script_class(resources, True)
+        self._corex_replacement[bossid] = self.get_script_class(pickup, True)
 
     def save_modifications(self, editor: PatcherEditor):
         editor.replace_asset("actors/items/randomizer_powerup/scripts/randomizer_powerup.lc", self._powerup_script)
